@@ -95,6 +95,15 @@ SECStatus PK11_GetTokenInfo(PK11SlotInfo *slot, CK_TOKEN_INFO *info);
 PRBool PK11_IsDisabled(PK11SlotInfo *slot);
 PRBool PK11_HasRootCerts(PK11SlotInfo *slot);
 PK11DisableReasons PK11_GetDisabledReason(PK11SlotInfo *slot);
+/* like PORT_Memcmp, return -1 if the version is less then the
+ * passed in version, 0 if it's equal to and 1 if it's greater than
+ * the passed in version. PKCS #11 returns versions in 2 places,
+ * once in the function table and once in the module. the former
+ * is good to determine if it is safe to call a new function,
+ * the latter is good for module functionality */
+PRInt32 PK11_CheckPKCS11Version(PK11SlotInfo *slot, CK_BYTE major,
+                                CK_BYTE minor, PRBool useFunctionTable);
+
 /* Prevents the slot from being used, and set disable reason to user-disable */
 /* NOTE: Mechanisms that were ON continue to stay ON */
 /*       Therefore, when the slot is enabled, it will remember */
@@ -241,6 +250,7 @@ int PK11_GetIVLength(CK_MECHANISM_TYPE type);
 SECItem *PK11_ParamFromIV(CK_MECHANISM_TYPE type, SECItem *iv);
 unsigned char *PK11_IVFromParam(CK_MECHANISM_TYPE type, SECItem *param, int *len);
 SECItem *PK11_BlockData(SECItem *data, unsigned long size);
+int PK11_GetMaxKeyLength(CK_MECHANISM_TYPE type);
 
 /* PKCS #11 to DER mapping functions */
 SECItem *PK11_ParamFromAlgid(SECAlgorithmID *algid);
@@ -424,6 +434,11 @@ PK11SymKey *PK11_PubDeriveWithKDF(SECKEYPrivateKey *privKey,
                                   CK_ATTRIBUTE_TYPE operation, int keySize,
                                   CK_ULONG kdf, SECItem *sharedData, void *wincx);
 
+/*
+ * Concatenate a pair of symmetric keys.
+ */
+PK11SymKey *PK11_ConcatSymKeys(PK11SymKey *left, PK11SymKey *right, CK_MECHANISM_TYPE target, CK_ATTRIBUTE_TYPE operation);
+
 /*
  * unwrap a new key with a symetric key.
  *  PK11_Unwrap returns a key which can do exactly one operation, and is
@@ -627,10 +642,20 @@ SECKEYPrivateKeyInfo *PK11_ExportPrivateKeyInfo(
     CERTCertificate *cert, void *wincx);
 SECKEYEncryptedPrivateKeyInfo *PK11_ExportEncryptedPrivKeyInfo(
     PK11SlotInfo *slot, SECOidTag algTag, SECItem *pwitem,
-    SECKEYPrivateKey *pk, int iteration, void *wincx);
+    SECKEYPrivateKey *pk, int iteration, void *pwArg);
 SECKEYEncryptedPrivateKeyInfo *PK11_ExportEncryptedPrivateKeyInfo(
     PK11SlotInfo *slot, SECOidTag algTag, SECItem *pwitem,
-    CERTCertificate *cert, int iteration, void *wincx);
+    CERTCertificate *cert, int iteration, void *pwArg);
+/* V2 refers to PKCS #5 V2 here. If a PKCS #5 v1 or PKCS #12 pbe is passed
+ * for pbeTag, then encTag and hashTag are ignored. If pbe is an encryption
+ * algorithm, then PKCS #5 V2 is used with prfTag for the prf. If prfTag isn't
+ * supplied prf will be SEC_OID_HMAC_SHA1 */
+SECKEYEncryptedPrivateKeyInfo *PK11_ExportEncryptedPrivKeyInfoV2(
+    PK11SlotInfo *slot, SECOidTag pbeTag, SECOidTag encTag, SECOidTag prfTag,
+    SECItem *pwitem, SECKEYPrivateKey *pk, int iteration, void *pwArg);
+SECKEYEncryptedPrivateKeyInfo *PK11_ExportEncryptedPrivateKeyInfoV2(
+    PK11SlotInfo *slot, SECOidTag pbeTag, SECOidTag encTag, SECOidTag prfTag,
+    SECItem *pwitem, CERTCertificate *cert, int iteration, void *pwArg);
 SECKEYPrivateKey *PK11_FindKeyByDERCert(PK11SlotInfo *slot,
                                         CERTCertificate *cert, void *wincx);
 SECKEYPublicKey *PK11_MakeKEAPubKey(unsigned char *data, int length);
@@ -715,6 +740,8 @@ PRBool PK11_FortezzaHasKEA(CERTCertificate *cert);
 CK_OBJECT_HANDLE PK11_FindEncodedCertInSlot(PK11SlotInfo *slot, SECItem *derCert, void *wincx);
 CK_OBJECT_HANDLE PK11_FindCertInSlot(PK11SlotInfo *slot, CERTCertificate *cert,
                                      void *wincx);
+CK_OBJECT_HANDLE PK11_FindObjectForCert(CERTCertificate *cert,
+                                        void *wincx, PK11SlotInfo **pSlot);
 SECStatus PK11_TraverseCertsForNicknameInSlot(SECItem *nickname,
                                               PK11SlotInfo *slot, SECStatus (*callback)(CERTCertificate *, void *),
                                               void *arg);
@@ -728,14 +755,8 @@ CK_BBOOL PK11_HasAttributeSet(PK11SlotInfo *slot,
                               PRBool haslock /* must be set to PR_FALSE */);
 
 /**********************************************************************
- *                   Hybrid Public Key Encryption  (draft-07)
+ *                   Hybrid Public Key Encryption
  **********************************************************************/
-/*
- * NOTE: All HPKE functions will fail with SEC_ERROR_INVALID_ALGORITHM
- * unless NSS is compiled with NSS_ENABLE_DRAFT_HPKE while spec (and
- * implementation) is in draft. The eventual RFC number is an input to
- * the key schedule, so applications opting into this MUST be prepared for
- * outputs to change when the implementation is updated or finalized. */
 
 /* Some of the various HPKE arguments would ideally be const, but the
  * underlying PK11 functions take them as non-const. To avoid lying to
@@ -766,6 +787,11 @@ void PK11_HPKE_DestroyContext(HpkeContext *cx, PRBool freeit);
 SECStatus PK11_HPKE_ExportContext(const HpkeContext *cx, PK11SymKey *wrapKey, SECItem **serialized);
 SECStatus PK11_HPKE_ExportSecret(const HpkeContext *cx, const SECItem *info, unsigned int L,
                                  PK11SymKey **outKey);
+
+/*
+ * Return a weak reference to SharedSecret stored in HPKE Context
+ */
+PK11SymKey *PK11_HPKE_GetSharedSecret(const HpkeContext *cx);
 const SECItem *PK11_HPKE_GetEncapPubKey(const HpkeContext *cx);
 HpkeContext *PK11_HPKE_ImportContext(const SECItem *serialized, PK11SymKey *wrapKey);
 SECStatus PK11_HPKE_Open(HpkeContext *cx, const SECItem *aad, const SECItem *ct, SECItem **outPt);
@@ -806,12 +832,58 @@ SECStatus PK11_VerifyWithMechanism(SECKEYPublicKey *key,
                                    const SECItem *param, const SECItem *sig,
                                    const SECItem *hash, void *wincx);
 
+/**********************************************************************
+ * Key Encapsulation Mechanisms
+ **********************************************************************/
+
+/*
+ * Using the given |pubKey|, generate a shared secret in |outKey| and an
+ * encapsulation of it in |ciphertext|. |outKey| will have usage attributes as
+ * specified in |attrFlags| and operation attributes as specified in |opFlags|.
+ *
+ * The function asserts that |pubKey|, |outKey|, and |ciphertext| are not NULL.
+
+ * If an error occurs, no allocations are made to |outKey| and |ciphertext|;
+ * otherwise (if SECSuccess is returned) allocations are made to |outKey| and
+ * |ciphertext| and the caller is responsible for freeing the memory occupied
+ * by these structures.
+ */
+SECStatus PK11_Encapsulate(SECKEYPublicKey *pubKey, CK_MECHANISM_TYPE target,
+                           PK11AttrFlags attrFlags, CK_FLAGS opFlags,
+                           PK11SymKey **outKey, SECItem **outCiphertext);
+
+/*
+ * Using the given |privKey|, decapsulate |ciphertext| and put the resulting
+ * shared secret in |outKey|. |outKey| will have usage attributes as specified
+ * in |attrFlags| and operation attributes as specified in |opFlags|.
+ *
+ * The function asserts that |privKey|, |ciphertext|, and |outKey| are not NULL.
+
+ * If an error occurs, |outKey| is not allocated; otherwise (if SECSuccess is
+ * returned) |outKey| is allocated and the caller is responsible for freeing
+ * the memory occupied by it.
+ */
+SECStatus
+PK11_Decapsulate(SECKEYPrivateKey *privKey, const SECItem *ciphertext,
+                 CK_MECHANISM_TYPE target, PK11AttrFlags attrFlags,
+                 CK_FLAGS opFlags, PK11SymKey **outKey);
+
 /**********************************************************************
  *                   Crypto Contexts
  **********************************************************************/
 void PK11_DestroyContext(PK11Context *context, PRBool freeit);
 PK11Context *PK11_CreateContextBySymKey(CK_MECHANISM_TYPE type,
-                                        CK_ATTRIBUTE_TYPE operation, PK11SymKey *symKey, SECItem *param);
+                                        CK_ATTRIBUTE_TYPE operation,
+                                        PK11SymKey *symKey,
+                                        const SECItem *param);
+PK11Context *PK11_CreateContextByPubKey(CK_MECHANISM_TYPE type,
+                                        CK_ATTRIBUTE_TYPE operation,
+                                        SECKEYPublicKey *pubKey,
+                                        const SECItem *param, void *pwArg);
+PK11Context *PK11_CreateContextByPrivKey(CK_MECHANISM_TYPE type,
+                                         CK_ATTRIBUTE_TYPE operation,
+                                         SECKEYPrivateKey *privKey,
+                                         const SECItem *param);
 PK11Context *PK11_CreateDigestContext(SECOidTag hashAlg);
 PK11Context *PK11_CloneContext(PK11Context *old);
 SECStatus PK11_DigestBegin(PK11Context *cx);
@@ -966,6 +1038,24 @@ SECStatus PK11_WriteRawAttribute(PK11ObjectType type, void *object,
 CK_OBJECT_HANDLE PK11_GetObjectHandle(PK11ObjectType objType, void *objSpec,
                                       PK11SlotInfo **slotp);
 
+/* PK11_ReadDistrustAfterAttribute reads either the
+ * CKA_NSS_SERVER_DISTRUST_AFTER or the CKA_NSS_EMAIL_DISTRUST_AFTER attribute
+ * from the specified object. The "CK_ATTRIBUTE_TYPE type" input must be one of
+ * these. If this function returns SECSuccess, then an attribute of the
+ * requested type was found and it was either:
+ *      (1) a single zero byte (indicating no distrust after date), or
+ *      (2) a valid 13 byte UTCTime.
+ * In case (1), the value *distrusted is set to PR_FALSE and the value *time
+ * is undefined. In case (2), the value *distrusted is set to PR_TRUE and the
+ * value *time is set by DER_UTCTimeToTime(). Neither *distrusted nor *time
+ * is defined if this function returns SECFailure.
+ */
+SECStatus PK11_ReadDistrustAfterAttribute(PK11SlotInfo *slot,
+                                          CK_OBJECT_HANDLE object,
+                                          CK_ATTRIBUTE_TYPE type,
+                                          /* out */ PRBool *distrusted,
+                                          /* out */ PRTime *time);
+
 /*
  * PK11_GetAllSlotsForCert returns all the slots that a given certificate
  * exists on, since it's possible for a cert to exist on more than one
@@ -1007,7 +1097,7 @@ PRBool SECMOD_HasRootCerts(void);
 /**********************************************************************
  * Other Utilities
  **********************************************************************/
-/* 
+/*
  * Get the state of the system FIPS mode -
  *  NSS uses this to force FIPS mode if the system bit is on. This returns
  *  the system state independent of the database state and can be called
@@ -1015,6 +1105,24 @@ PRBool SECMOD_HasRootCerts(void);
  */
 int SECMOD_GetSystemFIPSEnabled(void);
 
+/* FIPS indicator functions. Some operations are physically allowed, but
+ * are against the NSS FIPS security policy. This is because sometimes NSS
+ * functions are used in non-security contexts. You can call these functions
+ * to determine if you are operating inside or outside the the current vendor's
+ * FIPS Security Policy for NSS. NOTE: if the current version of NSS is not
+ * actually FIPS certified, then these functions will always return PR_FALSE */
+
+/* This function tells if if the last single shot operation on the slot
+ * was inside or outside the FIPS security policy */
+PRBool PK11_SlotGetLastFIPSStatus(PK11SlotInfo *slot);
+/* This tells you if the current operation is within the FIPS security policy. If
+ * you have called finalize on the context, it tells you if the last operation
+ * was within the FIPS security policy */
+PRBool PK11_ContextGetFIPSStatus(PK11Context *context);
+/* This tells you if the requested object was created in accordance to the
+ * NSS FIPS security policy. */
+PRBool PK11_ObjectGetFIPSStatus(PK11ObjectType objType, void *objSpec);
+
 SEC_END_PROTOS
 
 #endif
